/*
 * State.h
 *
 *  Created on: 21 Nov 2019
 *      Author: webpigeon
 */

#ifndef LIB_STATE_H_
#define LIB_STATE_H_

#include <vector>
#include <string>
#include <unordered_map>

#include "lib/hex.h"

using std::string;
using std::vector;
using std::unordered_map;

typedef Hex location;
typedef unordered_map<string, int32_t> property_map;

class terrain_type {
	public:
		int32_t get_properties(string const& name, int32_t default_val) const;

	private:
		property_map properties;
};

class entity_type {

	public:
		int32_t get_property(string const& name, int32_t default_val) const;

	private:
		entity_type* parent;
		property_map properties;
};

class entity {

	public:
		entity(entity_type* type): tag(0), pos(Hex(0,0,0)) {
			if (type == NULL){
				throw "cannot have a null type!";
			}
			this->type = type;
		}

		entity(entity const& rhs): tag(rhs.tag), pos(Hex(0,0,0)) {
			type = rhs.type;
		}

		int32_t get_property(string const& name) const;
		int32_t get_property(string const& name, int32_t val) const;
		void set_property(string const& name, int32_t val);

	private:
		entity_type* type;
		uint64_t tag;
		location pos;
		property_map properties;
};

struct data {
	terrain_type tile;
	int resource;
};

class state {
	public:
		state();
		virtual ~state();

		entity entity_id(uint32_t tag);
		entity entity_at(location location);
		vector<entity> entities_at(location location);
		void add_entity(entity entity);

		terrain_type terrain_at(location location);
		void set_terrain(location location, terrain_type t);

		uint32_t resource_at(location location);
		void set_resource(location location, uint32_t t);

		int32_t property_for(uint8_t player, const string& property) const;
		void set_property(uint8_t player, const string& property, int32_t val);

		vector<location> get_neighbours(location current);

	private:
		vector<entity> entities;
		unordered_map<location,data> map;
		unordered_map<uint8_t, property_map> properties;
		uint32_t turn;
};

#endif /* LIB_STATE_H_ */
